package com.rootcloud.common;

import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
public class Request {

    private final Logger log = LoggerFactory.getLogger(Request.class);

    /**
     * 4.0 需要修改密码返回的token
     */
    public static final String X_ROOTCLOUD_SETPW_TOKEN = "x-rootcloud-setpw-token";

    private static RestTemplate restTemplate = new RestTemplate();

    public ResponseEntity<String> requestForJP(String uri, HttpEntity<String> httpMethod) {
        return restTemplate.postForEntity(uri, httpMethod, String.class);
    }

    public JSONObject request(String uri, HttpMethod httpMethod, MultiValueMap<String, String> headers, Object t) {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headers != null) {
            headers.forEach((k, v) -> {
                httpHeaders.add(k, v.get(0));
            });
        }
        if (log.isDebugEnabled()) {
            log.debug("请求地址：{}，请求 参数：{} , 请求headers:{}", uri,
                    JSONObject.toJSONString(t), JSONObject.toJSONString(httpHeaders));
        }
        HttpEntity request = new HttpEntity<>(t, httpHeaders);
        ResponseEntity<JSONObject> responseEntity = restTemplate
                .exchange(uri, httpMethod, request, JSONObject.class);
        if (!responseEntity.getStatusCode().is2xxSuccessful()) {
            log.error("request error  result Code :{} , result: {}", responseEntity.getStatusCodeValue()
                    , JSONObject.toJSONString(responseEntity.getBody()));
            return null;
        }
        JSONObject jsonResult = responseEntity.getBody();
        if (log.isDebugEnabled()) {
            log.debug("请求返回: {}", JSONObject.toJSONString(jsonResult));
        }
        return jsonResult;
    }


    public JSONObject requestPost(String url, Map<String, String> map) {
        if (log.isDebugEnabled()) {
            log.debug("请求地址：{}，请求 参数：{} ", url, JSONObject.toJSONString(map));
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
        headers.setAccept(Stream.of(MediaType.APPLICATION_JSON_UTF8).collect(Collectors.toList()));
        HttpEntity<String> request = new HttpEntity<>(JSONObject.toJSONString(map), headers);
        try {
            ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(url, request, JSONObject.class);
            if (!responseEntity.getStatusCode().is2xxSuccessful()) {
                log.error("request error  result Code :{} , result: {}", responseEntity.getStatusCodeValue()
                        , JSONObject.toJSONString(responseEntity.getBody()));
                return null;
            }

            JSONObject jsonResult = responseEntity.getBody();
            if (log.isDebugEnabled()) {
                log.debug("请求返回: {}", JSONObject.toJSONString(jsonResult));
            }
            return jsonResult;
        } catch (HttpClientErrorException e) {
            log.error("请求发生异常", e);
            log.error("请求地址：{}，请求 参数：{} ", url, JSONObject.toJSONString(map));
            String bodyString = e.getResponseBodyAsString();
            if ("".equals(bodyString)) {
                return null;
            }
            log.error("异常返回结果:{}", bodyString);
            //IAM-1003	password need to be changed
            try {
                JSONObject object = JSONObject.parseObject(bodyString);
                String code = object.getString("code");
                //IAM-1003
                if ("IAM-1003".equals(code)) {
                    //4.0 出现这个错误码 说明密码需要重置。 返回xtoken 作为给前端 重置密码
                    HttpHeaders httpHeaders = e.getResponseHeaders();
                    String setpwToken = httpHeaders.getFirst(X_ROOTCLOUD_SETPW_TOKEN);
                    object.put(X_ROOTCLOUD_SETPW_TOKEN, setpwToken);
                    return object;
                }
            } catch (Exception e1) {
                log.error("异常结果转化异常", e1);
            }
        }
        return null;
    }


    /**
     * sany cas 是get 请求
     *
     * @param url
     * @param map
     * @return
     */
    public JSONObject requestCasGet(String url, Map<String, String> map) {
        if (log.isDebugEnabled()) {
            log.debug("请求地址：{}，请求 参数：{} ", url, JSONObject.toJSONString(map));
        }
        StringBuilder sb = new StringBuilder(url);
        if (map != null) {
            sb.append("?");
            for (Map.Entry<String, String> entry : map.entrySet()) {
                sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
            }
            sb.deleteCharAt(sb.length() - 1);
        }
        ResponseEntity<JSONObject> responseEntity = restTemplate.getForEntity(sb.toString(), JSONObject.class);
        if (!responseEntity.getStatusCode().is2xxSuccessful()) {
            log.error("request error  result Code :{} , result: {}", responseEntity.getStatusCodeValue()
                    , JSONObject.toJSONString(responseEntity.getBody()));
            return new JSONObject();
        }
        JSONObject jsonResult = responseEntity.getBody();
        if (log.isDebugEnabled()) {
            log.debug("请求返回: {}", JSONObject.toJSONString(jsonResult));
        }
        return jsonResult;
    }

    /**
     * 发送 application from 请求
     *
     * @param url
     * @param head
     * @param map
     * @return
     */
    public JSONObject requestCasPost(String url, Map<String, String> head, Map<String, String> map) {
        MultiValueMap<String, String> multiValueMap = new LinkedMultiValueMap<>();
        if (log.isDebugEnabled()) {
            log.debug("请求地址：{}，请求 参数：{} ,请求头部信息{}", url, JSONObject.toJSONString(map), head);
        }
        if (map != null) {
            for (Map.Entry<String, String> entry : map.entrySet()) {
                multiValueMap.add(entry.getKey(), entry.getValue());
            }
        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        if (head != null) {
            head.forEach((k, v) -> headers.add(k, v));
        }
        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(multiValueMap, headers);
        ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(url, request, JSONObject.class);
        if (!responseEntity.getStatusCode().is2xxSuccessful()) {
            log.error("request error  result Code :{} , result: {}", responseEntity.getStatusCodeValue()
                    , JSONObject.toJSONString(responseEntity.getBody()));
            return null;
        }
        JSONObject jsonResult = responseEntity.getBody();
        if (log.isDebugEnabled()) {
            log.debug("请求返回: {}", JSONObject.toJSONString(jsonResult));
        }
        return jsonResult;
    }


}
