package com.rootcloud.common.util;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class RestTemplateUtils {

    public static final RestTemplate restTemplate = new RestTemplate();

    /**
     * 发送get
     * @param url url
     * @param param 参数
     * @return response
     */
    public static JSONObject get(String url,Object param){
        return JSONObject.parseObject(restTemplate.getForObject(url, String.class, getJsonParam(param)));
    }
    /**
     * 发送post json参数
     * @param url url
     * @param param 参数
     * @return response
     */
    public static JSONObject postJson(String url,Object param){
        return JSONObject.parseObject(postString(url,param,MediaType.APPLICATION_JSON));
    }

    public static JSONObject postFrom(String url,Object param){
        return JSONObject.parseObject(postString(url,param,MediaType.APPLICATION_FORM_URLENCODED));
    }

    public static String postString(String url,Object param,MediaType content){
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(content);
        HttpEntity request;
        if (content.equals(MediaType.APPLICATION_JSON)) {
            request = new HttpEntity<>(getJsonParam(param), headers);
        } else if (content.equals(MediaType.APPLICATION_FORM_URLENCODED)){
            request = new HttpEntity<>(getParam(param), headers);
        }else{
            throw new RuntimeException("不支持的content-type类型");
        }
        return restTemplate.postForObject(url, request, String.class);
    }

    public static Map<String,Object> getJsonParam(Object param){
        MetaObject metaObject = SystemMetaObject.forObject(param);
        Map<String, Object> uriVariables = new HashMap<>();
        for (Field field : param.getClass().getDeclaredFields()) {
            Object value = metaObject.getValue(field.getName());
            if (value != null) {
                uriVariables.put(field.getName(),value);
            }
        }
        return uriVariables;
    }

    public static MultiValueMap<String,Object> getParam(Object param){
        MetaObject metaObject = SystemMetaObject.forObject(param);
        MultiValueMap<String, Object> uriVariables = new LinkedMultiValueMap<>();
        for (Field field : param.getClass().getDeclaredFields()) {
            Object value = metaObject.getValue(field.getName());
            if (value != null) {
                uriVariables.add(field.getName(),value);
            }
        }
        return uriVariables;
    }
    /**
     * GET 请求
     * @param url GET请求地址
     * @param responseType 返回对象类型
     * @param uriVariables url 参数键值对
     */
    public static <T> ResponseEntity<T> get(String url, Class<T> responseType, Map<String, ?> uriVariables) throws RestClientException {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        // 设置连接超时1分钟
        requestFactory.setConnectTimeout(60000);
        // 设置返回超时30秒
        requestFactory.setReadTimeout(30000);
        RestTemplate rest = new RestTemplate(requestFactory);
        UriComponentsBuilder uriComponentsBuilder = UriComponentsBuilder.fromHttpUrl(url);
        uriVariables.forEach(uriComponentsBuilder::queryParam);
        url = uriComponentsBuilder.build().encode().toString();
        log.info("REQ: {}", url);
        ResponseEntity<T> resp = rest.getForEntity(url, responseType);
        log.info("RESP: {}", resp);
        return resp;
    }

    /**
     * GET 请求 不转码
     * @param url GET请求地址
     * @param responseType 返回对象类型
     * @param uriVariables url 参数键值对
     */
    public static <T> ResponseEntity<T> getEncode(String url, Class<T> responseType, Map<String, ?> uriVariables) throws RestClientException {
        UriComponentsBuilder uriComponentsBuilder = UriComponentsBuilder.fromHttpUrl(url);
        uriVariables.forEach(uriComponentsBuilder::queryParam);
        return restTemplate.getForEntity(uriComponentsBuilder.build().toString(), responseType);
    }


}
