package com.rootcloud.config.auth;

import com.alibaba.fastjson.JSONObject;
import com.rootcloud.common.HttpClientUtil;
import com.rootcloud.common.IgnoreSecurity;
import com.rootcloud.common.constant.RoleIdConstant;
import com.rootcloud.common.enums.HttpRequestMethedEnum;
import com.rootcloud.domain.dto.EnterpriseInfoDTO;
import com.rootcloud.domain.entity.SiteDTO;
import com.rootcloud.domain.service.gbsj.JeCoreEnduserService;
import jline.internal.Log;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.AsyncHandlerInterceptor;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author: jun2.wang
 * @Date: 2021/1/21
 */
@Slf4j
public class AuthenticationInterceptor implements AsyncHandlerInterceptor {

    /**
     * token参数key
     */
    public static final String AUTHORIZATION = "authorization";
    public static final String LANGUAGE_APP = "lang";
    public static final String LANGUAGE_PC = "je-local-lang";

    private String authUrl;

    private RedisTemplate redisTemplate;

    private JeCoreEnduserService enduserService;

    public AuthenticationInterceptor(String authUrl, RedisTemplate redisTemplate, JeCoreEnduserService enduserService) {
        this.authUrl = authUrl;
        this.redisTemplate = redisTemplate;
        this.enduserService = enduserService;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws IOException {

        //解析语言
        String languageApp = this.getRequestValue(request, LANGUAGE_APP);
        String languagePc = this.getRequestValue(request, LANGUAGE_PC);
        if (isIgnoreSecurity(handler)) {
            initLanguage(languageApp,languagePc);
            return true;
        }
        //解析用户
        String authorization = this.getRequestValue(request, AUTHORIZATION);
        Cookie[] cookies = request.getCookies();
        StringBuilder cookieBuild = new StringBuilder();


        if (cookies != null && cookies.length > 0) {
            for (Cookie cookie : cookies) {
                cookieBuild.append(cookie.getName());
                cookieBuild.append("=");
                cookieBuild.append(cookie.getValue());
                cookieBuild.append(";");
            }
        }else {
            if (StringUtils.isNotBlank(authorization)) {
                String bearer = authorization.replaceFirst("Bearer ", "");
                cookieBuild.append("access_token=");
                cookieBuild.append(bearer);
            }
        }
        if (StringUtils.isBlank(authorization) && !cookieBuild.toString().contains("access_token")){
            log.warn("AuthenticationInterceptor:" + request.toString());
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return false;
        }
        Map<String, String> headers = new HashMap<>();
        headers.put("authorization", authorization);
        headers.put("Cookie", cookieBuild.toString());
        Object token = "";
        //判断缓存信息
        try {
            token = redisTemplate.opsForValue().get(authorization + "_" + cookieBuild.toString());
            if (token != null) {
                JSONObject jsonObject = JSONObject.parseObject(String.valueOf(token));
                //Log.info("登录验证拦截器获取缓存token租户id："+jsonObject.getString("tenantId"));
                UserContext.set(jsonObject.getString("userId"), jsonObject.getString("userName"), jsonObject.getString("tenantId"), false, false,authorization);
                initLanguage(languageApp,languagePc);
                return true;
            } else {
                String resp = HttpClientUtil.sendHttp(HttpRequestMethedEnum.HttpPost, authUrl, null, headers);
                JSONObject jsonObject = JSONObject.parseObject(resp);

                String userId= jsonObject.getString("userId");
                String zhid = enduserService.getTenantInfoNOSQL(userId);
                String tenantId = jsonObject.getString("zhId");
                //Log.info("拦截器UserId:"+userId+" 查询的租户ID："+zhid+" 获取token中的自带租户id:"+tenantId);
                if(StringUtils.isNotBlank(zhid)) {
                    jsonObject.put("tenantId",zhid);
                }
                redisTemplate.opsForValue().set(authorization + "_" + cookieBuild.toString(),jsonObject.toJSONString(), 600, TimeUnit.SECONDS);
                if (StringUtils.isNotBlank(jsonObject.getString("userId"))) {
                    UserContext.set(jsonObject.getString("userId"), jsonObject.getString("userName"), jsonObject.getString("tenantId"), false, false,authorization);
                }else {
                    log.warn("AuthenticationInterceptor:" + request.toString());
                    response.sendError(HttpServletResponse.SC_FORBIDDEN);
                    return false;
                }
                initLanguage(languageApp,languagePc);
                return true;
            }
        } catch (Exception e) {
            redisTemplate.delete(authorization + "_" + cookieBuild.toString());
            log.error("AuthenticationInterceptor:" + request.toString());
            e.printStackTrace();
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return false;
        }
    }

    //请求完成后清除用户信息
    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) {
        UserContext.remove();
    }

    void initLanguage(String langApp,String langPc){
        if ((StringUtils.isNotEmpty(langApp) && "en".equals(langApp)) || (StringUtils.isNotEmpty(langPc) && "en".equals(langPc))){
            UserContext.setLanguage("en");
        }else {
            UserContext.setLanguage("zh_CN");
        }
    }

    //初始化登录用户的信息
    LoginUserInfo initUser(String userId) {
        LoginUserInfo userInfo = enduserService.getUserInfo(userId);
        if (userInfo.getRoleIds().contains(RoleIdConstant.FWJL) || userInfo.getRoleIds().contains(RoleIdConstant.GCS)){
            StringBuilder sb = new StringBuilder();
            SiteDTO siteByUserId = enduserService.getSiteByUserId(userId);
            userInfo.setSiteId(siteByUserId.getSiteId());
            String[] strings = siteByUserId.getServiceArea().split(",");
            for (int i = 0; i < strings.length; i++) {
                String customs = enduserService.getCustomsBySiteAreaName(strings[i]);
                sb.append(customs);
                if (i<strings.length-1){
                    sb.append(",");
                }
            }
            userInfo.setCustomId(sb.toString());
        }else if (userInfo.getRoleIds().contains(RoleIdConstant.KHLD) || userInfo.getRoleIds().contains(RoleIdConstant.KH)){
            String custom = enduserService.getCustomsByUserId(userId);
            userInfo.setCustomId(custom);
        }
        return userInfo;
    }

    /**
     * 是否有权限
     *
     * @param handler
     * @return
     */
    private boolean isIgnoreSecurity(Object handler) {
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            // 获取方法上的注解
            IgnoreSecurity ignoreSecurity = handlerMethod.getMethod().getAnnotation(IgnoreSecurity.class);
            // 如果方法上的注解为空 则获取类的注解
            if (ignoreSecurity == null) {
                ignoreSecurity = handlerMethod.getMethod().getDeclaringClass().getAnnotation(IgnoreSecurity.class);
            }
            // 如果标记了注解，则判断权限
            return ignoreSecurity != null;
        }
        return true;
    }

    private String getRequestValue(HttpServletRequest request, String key) {
        String token = request.getParameter(key);
        if (StringUtils.isNotBlank(token)) {
            return token;
        }

        //获取请求头
        Enumeration<String> enumeration = request.getHeaders(key);
        if (enumeration.hasMoreElements()) {
            return enumeration.nextElement();
        }

        //获取cookie
        Cookie[] cookies = request.getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                String name = cookie.getName();
                if (key.equals(name)) {
                    return cookie.getValue();
                }
            }
        }
        return null;
    }

}
